#include <stdlib.h>
#include <assert.h>


size_t strlen(const char *str)
{
        const char *it = str;
        while (*it)
                ++it;

        return it - str;
}

/* return 0 when the strings are equal */
int strcmp(const char *s1, const char *s2)
{
        while(*s1 == *s2) {
                if (*s1 == '\0' /* => *s2 == '\0' */)
                        return 0;
                ++s1;
                ++s2;
        }

        return ((*s1 < *s2) ? -1 : 1);
}

int strncmp(const char *s1, const char *s2, size_t n)
{
        size_t i;
        for (i = 0; i < n; ++i) {
                if (s1[i] != s2[i])
                        return s1[i] < s2[i] ? -1 : 1;

                if (s1[i] == '\0' /* => s2[i] == '\0' */)
                        return 0;
        }

        return 0;
}

char *strcpy(char *dest, const char *src)
{
        int i = 0;

        while(src[i] != '\0') {
                dest[i] = src[i];
                ++i;
        }

        /* include the terminating 0 */
        dest[i] = '\0';

        return dest;
}

void *memcpy(void *dest, const void *src, size_t n)
{
        size_t i = 0;
        while(i < n) {
                ((char *)dest)[i] = ((char *)src)[i];
                ++i;
        }

        return dest;
}

char *strdup(const char *str)
{
        size_t len = strlen(str);
        char *mem = malloc(len);
        if (!mem)
                return ((void *) 0);

        memcpy(mem, str, len);
        return mem;
}

int main(int argc, char *argv[]) {
	char *str = strdup(argv[1]);
	char *str2 = strdup(argv[2]);

	/* check that the strings are the same */
	assert(strcmp(str, argv[1]));
	assert(strcmp(str2, argv[2]));

	if (strcmp(str, str2) != 0) {
		char *newstr = malloc(strlen(str) + strlen(str2) + 1);
		char *end = strcpy(newstr, str);
		assert(end == newstr + strlen(str));
		strcpy(end, str2);

		assert(strncmp(argv[1], newstr, strlen(str)) == 0);
		assert(strncmp(argv[2], newstr + strlen(str), strlen(str2)) == 0);
		free(str);
		free(str2);
	} else {
		char *newstr = malloc(2*strlen(str) + 1);
		char *end = strcpy(newstr, str);
		assert(end == newstr + strlen(str));
		strcpy(end, str);

		assert(strncmp(argv[1], newstr, strlen(str)) == 0);
		assert(strncmp(argv[1], newstr + strlen(str), strlen(str)) == 0);
		free(str);
	}

	return 0;
}

